package rule

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/lib/comm_plan"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/svc"
	"context"
	"errors"
	"golang.org/x/sync/errgroup"
	"sync"
	"xorm.io/xorm"
)

// BatchGetCommission 批量分佣 商品显示
func BatchGetCommissionByGoods(eg *xorm.Engine, dbName string, commissionParamList []*md.CommissionFirstParam) (map[string]*comm_plan.LvUser, error) {
	var (
		isShare          = false
		isAllLevelReturn = false
	)
	goodsId2lvUser := make(map[string]*comm_plan.LvUser, len(commissionParamList))
	if len(commissionParamList) == 0 {
		return goodsId2lvUser, errors.New("参数错误")
	}

	group, _ := errgroup.WithContext(context.Background())
	var mu sync.Mutex
	virCfg, _ := db.SysCfgGetOne(eg, "virtual_coin_rebate_type")
	var virType = "price"
	if virCfg != nil && virCfg.Val != "" {
		virType = virCfg.Val
	}
	for _, param := range commissionParamList {
		param := param // 为下面的闭包创建局部变量
		group.Go(func() error {
			if param.IsShare != 0 {
				isShare = true
			}

			if param.IsAllLevelReturn != 0 {
				isAllLevelReturn = true
			}

			if param.GoodsId == "" {
				return errors.New("商品ID缺失")
			}

			_, _, _, _, lvUser, err := svc.GetRewardCommission(eg, &param.CommissionParam, isShare, param.Uid, param.Provider, dbName, isAllLevelReturn, map[string]string{}, virType)
			if err != nil {
				return err
			}
			mu.Lock()
			goodsId2lvUser[param.GoodsId] = lvUser
			mu.Unlock()

			return nil
		})
	}
	if err := group.Wait(); err != nil {
		return goodsId2lvUser, errors.New("处理错误")
	}

	return goodsId2lvUser, nil
}

// BatchGetCommission 批量分佣 订单显示
func BatchGetCommissionByOrder(engine *xorm.Engine, dbName string, CommissionParamList []*md.CommissionFirstParam) (map[string]*comm_plan.SettleCommissionToGuide, error) {
	var (
		isShare          = false
		isAllLevelReturn = false
	)
	goodsId2lvUser := make(map[string]*comm_plan.SettleCommissionToGuide, len(CommissionParamList))
	if len(CommissionParamList) == 0 {
		return goodsId2lvUser, errors.New("参数缺失")
	}

	group, _ := errgroup.WithContext(context.Background())
	var mu sync.Mutex
	virCfg, _ := db.SysCfgGetOne(engine, "virtual_coin_rebate_type")
	var virType = "price"
	if virCfg != nil && virCfg.Val != "" {
		virType = virCfg.Val
	}
	for _, param := range CommissionParamList {
		param := param // 为下面的闭包创建局部变量
		group.Go(func() error {
			if param.IsShare != 0 {
				isShare = true
			}
			if param.IsAllLevelReturn != 0 {
				isAllLevelReturn = true
			}

			if param.Oid == "" {
				return errors.New("订单缺失")
			}
			param.CommissionParam.OldPrice = param.CommissionParam.GoodsPrice

			profit, pvdFee, sysFee, subsidyFee, lvUser, err := svc.GetRewardCommission(engine, &param.CommissionParam, isShare, param.Uid, param.Provider, dbName, isAllLevelReturn, map[string]string{}, virType)
			if err != nil {
				return err
			}
			mu.Lock()
			goodsId2lvUser[param.Oid] = &comm_plan.SettleCommissionToGuide{
				Profit:     profit,
				PvdFee:     pvdFee,
				SysFee:     sysFee,
				SubsidyFee: subsidyFee,
				LvUser:     lvUser,
			}
			mu.Unlock()

			return nil
		})
	}
	if err := group.Wait(); err != nil {
		return goodsId2lvUser, err
	}

	return goodsId2lvUser, nil
}
